@@ -3,7 +3,7 @@ package Protocol::WebSocket::Frame;
 use strict;
 use warnings;
 
-use Encode;
+use Encode ();
 use Scalar::Util 'readonly';
 
 sub new {
@@ -14,7 +14,14 @@ sub new {
     my $self = {@_};
     bless $self, $class;
 
-    $self->{buffer} = defined $buffer ? $buffer : '';
+    $buffer = '' unless defined $buffer;
+
+    if (Encode::is_utf8($buffer)) {
+        $self->{buffer} = Encode::encode('UTF_8', $buffer);
+    }
+    else {
+        $self->{buffer} = $buffer;
+    }
 
     return $self;
 }
@@ -33,15 +40,30 @@ sub append {
 sub next {
     my $self = shift;
 
+    my $bytes = $self->next_bytes;
+    return unless defined $bytes;
+
+    return Encode::decode('UTF-8', $bytes);
+}
+
+sub next_bytes {
+    my $self = shift;
+
     return unless $self->{buffer} =~ s/^[^\x00]*\x00(.*?)\xff//s;
 
-    return Encode::decode_utf8($1);
+    return $1;
+}
+
+sub to_bytes {
+    my $self = shift;
+
+    return "\x00" . $self->{buffer} . "\xff";
 }
 
 sub to_string {
     my $self = shift;
 
-    return "\x00" . Encode::encode_utf8($self->{buffer}) . "\xff";
+    return "\x00" . Encode::decode('UTF-8', $self->{buffer}) . "\xff";
 }
 
 1;
@@ -71,7 +93,8 @@ Construct or parse a WebSocket frame.
 
 =head2 C<new>
 
-Create a new L<Protocol::WebSocket::Frame> instance.
+Create a new L<Protocol::WebSocket::Frame> instance. Automatically detect if the
+passed data is a Perl string or bytes.
 
 =head2 C<append>
 
@@ -85,16 +108,24 @@ Append a frame chunk.
     $frame->append("\x00foo");
     $frame->append("\xff\x00bar\xff");
 
-    $fram->next; # foo
-    $fram->next; # bar
+    $frame->next; # foo
+    $frame->next; # bar
 
-Return the next frame.
+Return the next frame as a Perl string.
+
+=head2 C<next_bytes>
+
+Return the next frame as a UTF-8 encoded string.
 
 =head2 C<to_string>
 
     my $frame = Protocol::WebSocket::Frame->new('foo');
     $frame->to_string; # \x00foo\xff
 
-Construct a WebSocket frame.
+Construct a WebSocket frame as a Perl string.
+
+=head2 C<to_bytes>
+
+Construct a WebSocket frame as a UTF-8 encoded string.
 
 =cut
